using System;
using System.Linq;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.RuntimeDetour;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;


namespace Bitsbolts.Entities {
  static class Commands {
    [Command("bb_flag", "[bits & bolts] Gets or sets session flags.\nUsage: bb_flag, bb_flag [flag], bb_flag [flag] [0/1/true/false]")]
    private static void OnCommandFlag(string flag, string setstr) {
      if (Engine.Scene is Level level) {
        if (flag == null) {
          string output = "";
          foreach (var iter_flag in level.Session.Flags) {
            if (output != "") { output += ", "; }
            output += iter_flag;
          }
          Engine.Commands.Log(output);
        } else if (setstr == null) {
          Engine.Commands.Log($"flag {flag} is {level.Session.GetFlag(flag) ? "true" : "false"}");
        } else {
          var set = setstr == "1" || setstr.ToLower() == "true";
          level.Session.SetFlag(flag, set);
          Engine.Commands.Log($"flag {flag} is set to {set ? "true" : "false"}");
        }
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }

    [Command("bb_levelflag", "[bits & bolts] Gets or sets level flags.\nUsage: bb_levelflag, bb_levelflag [flag], bb_levelflag [flag] [0/1/true/false]")]
    private static void OnCommandLevelFlag(string flag, string setstr) {
      if (Engine.Scene is Level level) {
        if (flag == null) {
          string output = "";
          foreach (var iter_flag in level.Session.LevelFlags) {
            if (output != "") { output += ", "; }
            output += iter_flag;
          }
          Engine.Commands.Log(output);
        } else if (setstr == null) {
          Engine.Commands.Log($"flag {flag} is {level.Session.GetLevelFlag(flag) ? "true" : "false"}");
        } else {
          var set = setstr == "1" || setstr.ToLower() == "true";

          if (set) {
            level.Session.LevelFlags.Add(flag);
          } else {
            level.Session.LevelFlags.Remove(flag);
          }
          Engine.Commands.Log($"level flag {flag} is set to {set ? "true" : "false"}");
        }
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }

    [Command("bb_saveflag", "[bits & bolts] Gets or sets save data flags.\nCareful, this may affect other maps on the same savefile!\nUsage: bb_saveflag, bb_saveflag [flag], bb_saveflag [flag] [0/1/true/false]")]
    private static void OnCommandSaveDataFlag(string flag, string setstr) {
      if (SaveData.Instance != null) {
        var savedata = SaveData.Instance;
        if (flag == null) {
          string output = "";
          foreach (var iter_flag in savedata.Flags) {
            if (output != "") { output += ", "; }
            output += iter_flag;
          }
          Engine.Commands.Log(output);
        } else if (setstr == null) {
          Engine.Commands.Log($"save data flag {flag} is {savedata.HasFlag(flag) ? "true" : "false"}");
        } else {
          var set = setstr == "1" || setstr.ToLower() == "true";

          if (set) {
            savedata.SetFlag(flag);
          } else {
            savedata.Flags.Remove(flag);
          }
          Engine.Commands.Log($"save data flag {flag} is set to {set ? "true" : "false"}");
        }
      } else {
        Engine.Commands.Log("no save file is loaded dumbass", Color.Yellow);
      }
    }

    [Command("bb_dnlflag", "[bits & bolts] Gets or sets do not load flags.\nUsage: bb_dnlflag, bb_dnlflag [level:id], bb_dnlflag [level:id] [0/1/true/false]")]
    private static void OnCommandDoNotLoadFlag(string flag, string setstr) {
      if (Engine.Scene is Level level) {
        if (flag == null) {
          string output = "";
          foreach (var iter_flag in level.Session.DoNotLoad) {
            if (output != "") { output += ", "; }
            output += iter_flag;
          }
          Engine.Commands.Log(output);
          return;
        }

        var eid = new EntityID();
        try {
          eid = Parsing.ParseEID(flag);
        } catch (Parsing.ParseException) {
          Engine.Commands.Log("not a valid entityid dumbass. use format: levelname:id", Color.Yellow);
        }

        if (setstr == null) {
          Engine.Commands.Log($"do not load flag {flag} is {level.Session.DoNotLoad.Contains(eid) ? "true" : "false"}");
        } else {
          var set = setstr == "1" || setstr.ToLower() == "true";

          if (set) {
            level.Session.DoNotLoad.Add(eid);
          } else {
            level.Session.DoNotLoad.Remove(eid);
          }
          Engine.Commands.Log($"do not load flag {flag} is set to {set ? "true" : "false"}");
        }
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }

    [Command("bb_entityflag", "[bits & bolts] Gets or removes session entity flags.\nUsage: bb_entityflag, bb_entityflag [level:id], bb_entityflag [level:id] [0/false]")]
    private static void OnCommandSavedEntityFlag(string flag, string setstr) {
      if (Engine.Scene is Level) {
        var entities = MainModule.Session.StoredEntities;
        if (flag == null) {
          string output = "";
          foreach (var iter_flag in entities) {
            if (output != "") { output += ", "; }
            output += iter_flag;
          }
          Engine.Commands.Log(output);
          return;
        }

        var eid = new EntityID();
        try {
          eid = Parsing.ParseEID(flag);
        } catch (Parsing.ParseException) {
          Engine.Commands.Log("not a valid entityid dumbass. use format: levelname:id", Color.Yellow);
        }

        if (setstr == null) {
          Engine.Commands.Log($"session entity flag {flag} is {entities.ContainsKey(eid) ? "true" : "false"}");
        } else {
          var set = setstr == "1" || setstr.ToLower() == "true";

          if (set) {
            Engine.Commands.Log("setting session entities is like unsupported man", Color.Yellow);
            return;
          } else {
            entities.Remove(eid);
            Engine.Commands.Log($"session entity flag {flag} is set to false");
          }
        }
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }

    [Command("bb_give_jelly", "[bits & bolts] Spawns in an immortal jellyfish")]
    private static void OnCommandGiveJelly() {
      if (Engine.Scene is Level level) {
        var player = level.Tracker.GetEntity<Player>();
        if (player == null) { return; }

        var jelly = new Glider(player.Position + new Vector2(0f, -16f), true, false);
        level.Add(jelly);
        jelly.AddTag(Tags.TransitionUpdate | Tags.Global);
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }

    [Command("bb_ent", "[bits & bolts] Spawns in an entity. Usage: bb_ent [ModName/EntityName]")]
    private static void OnCommandSpawn(string ent) {
      if (ent == null) {
        Engine.Commands.Log("provide a entity id ModName/EntityName", Color.Yellow);
        return;
      }

      if (Engine.Scene is Level level) {
        var player = level.Tracker.GetEntity<Player>();
        if (player == null) { return; }

        string matchedName = Level.EntityLoaders.Keys.Where(
            name => name.Split(new char[] {'/'}, System.StringSplitOptions.RemoveEmptyEntries)
              .Last()
              .ToLower().Replace(" ", "").Replace("/", "").Replace("-", "").Replace("_", "")
              .Contains(ent.ToLower())
          ).OrderBy(
            name => name.Split(new char[] {'/'}, System.StringSplitOptions.RemoveEmptyEntries)
              .Last()
              .Replace(" ", "").Replace("/", "").Replace("-", "").Replace("_", "")
              .Length
          ).FirstOrDefault();

        if (matchedName == null) {
          Engine.Commands.Log("could not find named entity :(", Color.Yellow);
          return;
        }

        var data = new EntityData {
          Position = player.Position + new Vector2(0f, -24f) - level.LevelOffset,
          Origin = player.Position + new Vector2(0f, -24f) - level.LevelOffset,
          ID = Calc.Random.Next(),
          Name = matchedName,
          Height = 24,
          Width = 24,
          Level = level.Session.LevelData,
          Values = new Dictionary<string, object>(),
          Nodes = new Vector2[0],
        };

        try {
          if (!Level.LoadCustomEntity(data, level)) {
            Engine.Commands.Log("could not find named entity :(", Color.Yellow);
          }
        } catch {
          Engine.Commands.Log("loading entity failed :(", Color.Yellow);
        }
      } else {
        Engine.Commands.Log("not in a map dumbass", Color.Yellow);
      }
    }
  }
}
