using System;
using System.Collections;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts.Components {
  internal class AfterUpdateCoroutine : Component {
    private AfterUpdateHook afterUpdateHook;

    internal bool Finished { get; private set; }
    internal bool RemoveOnComplete = true;
    internal bool UseRawDeltaTime = false;
    private Stack<IEnumerator> enumerators;
    private float waitTimer = 0f;
    private bool ended = false;


    internal AfterUpdateCoroutine(IEnumerator routine, bool removeOnComplete = true) : base(true, false) {
      this.enumerators = new Stack<IEnumerator>();
      this.enumerators.Push(routine);
      this.RemoveOnComplete = removeOnComplete;
    }

    internal AfterUpdateCoroutine(bool removeOnComplete = true) : base(false, false) {
      this.enumerators = new Stack<IEnumerator>();
      this.RemoveOnComplete = removeOnComplete;
    }

    internal void Cancel() {
      this.Active = false;
      this.Finished = true;
      this.waitTimer = 0f;
      this.enumerators.Clear();
      this.ended = true;
    }

    internal void Replace(IEnumerator functionCall) {
      this.Active = true;
      this.Finished = false;
      this.waitTimer = 0f;
      this.enumerators.Clear();
      this.enumerators.Push(functionCall);
      this.ended = true;
    }

    public override void Added(Entity entity) {
      base.Added(entity);
      entity.Add(afterUpdateHook = new AfterUpdateHook(this.AfterUpdate));
    }

    public override void Removed(Entity entity) {
      if (afterUpdateHook != null) {
        afterUpdateHook.RemoveSelf();
        afterUpdateHook = null;
      }
      base.Removed(entity);
    }

    public void AfterUpdate() {
      this.ended = false;
      if (this.waitTimer > 0f) {
        this.waitTimer -= (this.UseRawDeltaTime ? Engine.RawDeltaTime : Engine.DeltaTime);
        return;
      }

      if (this.enumerators.Count > 0) {
        IEnumerator enumerator = this.enumerators.Peek();
        if (enumerator.MoveNext() && !this.ended) {
          if (enumerator.Current is int idelay) {
            this.waitTimer = (float) idelay;
          } else if (enumerator.Current is float fdelay) {
            this.waitTimer = (float) fdelay;
            return;
          } else if (enumerator.Current is IEnumerator inner) {
            this.enumerators.Push(inner);
            return;
          }
        } else if (!this.ended) {
          this.enumerators.Pop();
          if (this.enumerators.Count == 0) {
            this.Finished = true;
            this.Active = false;
            if (this.RemoveOnComplete) {
              afterUpdateHook.RemoveSelf();
              base.RemoveSelf();
            }
          }
        }
      }
    }
  }
}
