using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts.Components {
  // This class is effectively internal but Monocle [Tracked]
  [Tracked]
  public class AfterUpdateHook : Component {
    [OnLoad]
    internal static void Load() {
      On.Monocle.Scene.AfterUpdate += OnSceneAfterUpdate;
    }

    [OnUnload]
    internal static void Unload() {
      On.Monocle.Scene.AfterUpdate -= OnSceneAfterUpdate;
    }


    internal Action Callback;
    internal AfterUpdateHook(Action callback) : base(true, false) {
      this.Callback = callback;
    }

    private static void OnSceneAfterUpdate(On.Monocle.Scene.orig_AfterUpdate orig, Scene scene) {
      // Triggered outside of level, potentially after using live mod installer
      // when tracker is still uninitialized
      if (!(scene is Level)) {
        orig(scene);
        return;
      }

      foreach (AfterUpdateHook hook in scene.Tracker.GetComponentsCopy<AfterUpdateHook>()) {
        if (
          hook.Entity != null && hook.Entity.Active && hook.Active
          && (!scene.Paused || hook.Entity.TagCheck(Tags.PauseUpdate))
          && (!(scene as Level).Transitioning || hook.Entity.TagCheck(Tags.TransitionUpdate))
          && (!(scene as Level).Frozen || hook.Entity.TagCheck(Tags.FrozenUpdate | Tags.PauseUpdate))
        ) { hook.Callback(); }
      }

      orig(scene);
    }
  }
}
