using System;
using System.Collections;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;


namespace Bitsbolts.Components {
  public class ControllingPlayback : Component {
    public readonly List<Player.ChaserState> Timeline;
    public int Frame;
    public float Time;
    public Vector2 Origin;

    public ControllingPlayback(List<Player.ChaserState> timeline) : base(true, true) {
      this.Timeline = timeline;
    }

    public override void Added(Entity entity) {
      base.Added(entity);
      entity.Add(new AfterUpdateCoroutine(this.PlaybackRoutine()));
    }

    public IEnumerator PlaybackRoutine() {
      this.Origin = base.Entity.Position;
      this.SetFrame(this.Frame);

      while (true) {
        var newFrame = this.Timeline.IndexOf(state => this.Time < state.TimeStamp);
        if (newFrame == -1) {
          this.SetFrame(this.Timeline.Count - 1);
          break;
        }

        newFrame = newFrame == 0 ? 0 : newFrame - 1;
        if (newFrame >= this.Timeline.Count) { break; }

        this.SetFrame(newFrame);

        yield return null;
        this.Time += Engine.DeltaTime;
      }

      base.RemoveSelf();
      yield break;
    }

    public void SetFrame(int frame) {
      var prevState = this.Timeline[this.Frame];
      var state = this.Timeline[frame];

      var player = base.Entity as Player;
      PlayerSprite sprite = base.Entity.Get<PlayerSprite>();
      PlayerHair hair = base.Entity.Get<PlayerHair>();

      base.Entity.Position = this.Origin + state.Position;

      // don't flip the sprite! that's what Player.facing is for
      sprite.Scale = state.Scale.Abs();
      if (sprite.Scale.X != 0) {
        hair.Facing = (Facings) Math.Sign(sprite.Scale.X);
        player.Facing = hair.Facing;
      }

      hair.Color = state.HairColor;
      if (sprite.Mode == PlayerSpriteMode.Playback) {
        sprite.Color = state.HairColor;
      }

      string prevAnimation = sprite.CurrentAnimationID;
      if (state.Animation != prevAnimation && sprite.Has(state.Animation)) {
        sprite.Play(state.Animation, restart: true);
      }

      player.DashDir = state.DashDirection;
      // todo: set player states maybe?
      // todo: sync jump and dash player properties (count, timers, etc.)
      // fixme: ground landing particles are broke asf
      // todo: implement dash gfx
      // todo: implement sfx for everything >:(

      this.Frame = frame;
      // hair.AfterUpdate();
    }
  }
}
