using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;

using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Components {
  // This class is effectively internal but Monocle [Tracked]
  [Tracked]
  public class FlagstrCollidable : Component {
    private Flagstrs flags;
    internal Action<bool> OnChange;
    internal FlagstrListener Listener;

    internal FlagstrCollidable(Flagstrs flags) : base(false, false) {
      this.flags = flags;
      this.OnChange = (_) => {};
    }

    internal FlagstrCollidable(Flagstrs flags, Action<bool> onChange) : base(false, false) {
      this.flags = flags;
      this.OnChange = onChange;
    }

    public override void Added(Entity entity) {
      base.Added(entity);
      entity.Add(this.Listener = new FlagstrListener {
        Flags = this.flags, TriggerOnAwake = true, OnAll = this.OnAllFlags
      });
    }

    private void OnAllFlags(bool enabled) {
      if (base.Entity != null) {
        base.Entity.Collidable = enabled;
      }

      this.OnChange(enabled);
    }

    public override void Removed(Entity entity) {
      // if this component gets removed, also remove the now useless listener
      if (this.Listener != null) {
        this.Listener.RemoveSelf();
        this.Listener = null;
      }

      base.Removed(entity);
    }
  }
}
