using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using Celeste.Mod;

using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


// Thank you Frost Helper my beloved

namespace Bitsbolts.Components {
  // This class is effectively internal but Monocle [Tracked]
  [Tracked]
  public class FlagstrListener : Component {
    [OnLoad]
    public static void Load() {
      On.Celeste.Session.SetFlag += OnSessionSetFlag;
    }

    [OnUnload]
    public static void Unload() {
      On.Celeste.Session.SetFlag -= OnSessionSetFlag;
    }


    internal Flagstrs Flags;
    internal bool TriggerOnAwake = true;

    internal Action<string, bool> OnChange = null;
    private bool prevAll;
    internal Action<bool> OnAll = null;
    private bool prevAny;
    internal Action<bool> OnAny = null;


    internal FlagstrListener() : base(false, false) {}
    internal FlagstrListener(Flagstrs flags) : base(false, false) { this.Flags = flags; }
    internal FlagstrListener(Flagstrs flags, bool triggerOnAwake) : base(false, false) {
      this.Flags = flags;
      this.TriggerOnAwake = triggerOnAwake;
    }

    public override void EntityAwake() {
      base.EntityAwake();

      var flags = (base.Scene as Level).Session.GetFlagstrs(this.Flags);
      this.prevAll = flags.All(f => f.Value);
      this.prevAny = flags.Any(f => f.Value);

      if (this.TriggerOnAwake) {
        if (this.OnAll != null) { this.OnAll(this.prevAll); }
        if (this.OnAny != null) { this.OnAny(this.prevAny); }
      }
    }

    private static void OnSessionSetFlag(
      On.Celeste.Session.orig_SetFlag orig, Session self, string changeFlag, bool setTo
    ) {
      var prevSet = self.GetFlag(changeFlag);
      orig(self, changeFlag, setTo);

      if (prevSet == setTo) { return; }

      foreach (
        FlagstrListener listener in Engine.Scene.Tracker.GetComponents<FlagstrListener>()
      ) {
        if (!listener.Flags.ContainsKey(changeFlag)) { continue; }

        if (listener.OnChange != null) { listener.OnChange(changeFlag, setTo); }

        // no point in generating flagstrs
        if (listener.OnAll == null && listener.OnAny == null) { continue; }

        var flags = self.GetFlagstrs(listener.Flags);

        if (listener.OnAll != null) {
          var newAll = flags.All(f => f.Value);
          if (listener.prevAll != newAll) { listener.OnAll(newAll); }
          listener.prevAll = newAll;
        }

        if (listener.OnAny != null) {
          var newAny = flags.Any(f => f.Value);
          if (listener.prevAny != newAny) { listener.OnAny(newAny); }
          listener.prevAny = newAny;
        }
      }
    }
  }
}
