using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;

using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Components {
  // This class is effectively internal but Monocle [Tracked]
  [Tracked]
  public class FlagstrTrigger : Component {
    private Flagstrs flags;
    internal Action<bool> OnChange;
    internal FlagstrCollidable Listener;

    internal FlagstrTrigger(Flagstrs flags) : base(false, false) {
      this.flags = flags;
      this.OnChange = (_) => {};
    }

    internal FlagstrTrigger(Flagstrs flags, Action<bool> onChange) : base(false, false) {
      this.flags = flags;
      this.OnChange = onChange;
    }

    public override void Added(Entity entity) {
      base.Added(entity);
      entity.Add(this.Listener = new FlagstrCollidable(this.flags, this.OnCollidableChange));
    }

    public void OnCollidableChange(bool collidable) {
      var trigger = base.Entity as Trigger;
      if (!collidable && trigger.Triggered) {
        // if the player is inside the trigger, then we should to untrigger it first
        Player player = base.Scene.Tracker.GetEntity<Player>();
        if (player != null) { trigger.OnLeave(player); }

        // unsetting Triggered means that when this trigger is collideable again,
        // it will retrigger OnEnter, even if the player never actually left
        trigger.Triggered = false;
      }
      this.OnChange(collidable);
    }

    public override void Removed(Entity entity) {
      // if this component gets removed, also remove the now useless listener
      if (this.Listener != null) {
        this.Listener.RemoveSelf();
        this.Listener = null;
      }

      base.Removed(entity);
    }
  }
}
