using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts.Components {
  // So due to a Level.TransitionRoutine quirk, it's possible to pause before
  // the first Update happens after the end of a transition, which means
  // triggers don't actually get triggered until the end of the transition.
  // This bug allows players to pause before Update gets called, which can ruin
  // certain triggers & entities. This fixes that by checking player collision
  // after the transition ends but before Level.CanPause gets changed.
  public class TransitionTrigger : Component {
    private bool before;
    private TransitionListener listener;
    public TransitionTrigger(bool before) : base(false, false) {
      this.before = before;
    }

    public override void Added(Entity entity) {
      if (before) {
        this.listener = new TransitionListener { OnInBegin = this.Check };
      } else {
        this.listener = new TransitionListener { OnInEnd = this.Check };
      }
      entity.Add(this.listener);
    }

    public override void Removed(Entity entity) {
      if (this.listener != null) {
        entity.Remove(this.listener);
        this.listener = null;
      }
    }

    private void Check() {
      if (base.Entity == null) { return; }

      var trigger = base.Entity as Trigger;
      Player player = base.Entity.CollideFirst<Player>();
      if (player != null && !trigger.PlayerIsInside && !trigger.Triggered) {
        trigger.Triggered = true;
        trigger.OnEnter(player);
        trigger.Triggered = false;
      }
    }
  }
}
