using System;
using System.Reflection;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.RuntimeDetour;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;


namespace Bitsbolts.Components {
  [Tracked]
  public class UnboundedDashing : Component {
    private static IDetour ILPlayerDashCoroutineHook;

    [OnLoad]
    public static void Load() {
      // fixme: this component doesn't work for vertical velocity wtf?
      // todo: check if we broke wavedashing or not...

      // Bounce??? doesn't mod speed?
      // WallJump in the edge case the player is jumping off a jump thru at fast speeds lol, or if moving vertically fast

      // something for boosters?
      // RedDashCoroutine maybeee?

      // WallJump

      // ReflectBounce??
      // DreamDashBegin???
      // SideBounce???
      // SuperBounce???
      // SuperWallJump
      // SuperJump

      ILPlayerDashCoroutineHook = new ILHook(
        typeof(Player).GetMethod("DashCoroutine", BindingFlags.NonPublic | BindingFlags.Instance)
          .GetStateMachineTarget(),
        ILPlayerDashCoroutine
      );

      IL.Celeste.Player.SuperJump += ILPlayerSuperJump;  // supers/hypers
      IL.Celeste.Player.DreamDashBegin += ILPlayerDreamDashBegin;  // uncap dream speed

      // IL.Celeste.Player.WallJump += ILPlayerWallJump;  // wounces
      IL.Celeste.Player.SuperWallJump += ILPlayerSuperWallJump;  // wounces

      IL.Celeste.Player.SuperBounce += ILPlayerSuperBounce;  // upwards springs
      IL.Celeste.Player.SideBounce += ILPlayerSideBounce;  // side springs

      // IL.Celeste.Player.ReflectBounce += ILPlayerReflectBounce;  // assist dream bounce
      // IL.Celeste.Player.ExplodeLaunch += ILPlayerExplodeLaunch;  // puffers?
    }

    [OnUnload]
    public static void Unload() {
      ILPlayerDashCoroutineHook.Dispose();
    }



    public UnboundedDashing() : base(false, false) {}

    public static Vector2 CombineDash(Vector2 dash, Vector2 speed, float minSpeed) {
      if (speed.Length() <= minSpeed + .1f) { return dash; }

      var projected = Util.Project(dash, speed);

      if (projected.Length() <= minSpeed + .1f) { return dash; }
      return projected;
    }


    private static void ILPlayerDashCoroutine(ILContext il) {
      var cursor = new ILCursor(il);

      if (!cursor.TryGotoNext(MoveType.Before, instr => instr.MatchStfld<Player>("Speed"))) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find stfld this.Speed in Player.DashCoroutine.");
        return;
      }

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.DashCoroutine to mod this.Speed!");

      // google black magic
      // holy hell!
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.Emit(
        OpCodes.Ldfld,
        typeof(Player).GetMethod("DashCoroutine", BindingFlags.NonPublic | BindingFlags.Instance)
          .GetStateMachineTarget()
          .DeclaringType.GetField("<>4__this")
      );
      cursor.EmitDelegate<Func<Vector2, Player, Vector2>>(
        (newSpeed, player) => {
          if (player.Get<UnboundedDashing>() == null) { return newSpeed; }

          // var dd = DynamicData.For(player);
          // var beforeDashSpeed = dd.Get<Vector2>("beforeDashSpeed");
          // if (newSpeed.X == beforeDashSpeed.X) {
          //   var speed = this.CorrectDashPrecision(vector);
          //   return new Vector2(newSpeed.X)
          // }
          return UnboundedDashing.CombineDash(newSpeed, player.Speed, 240f);
        }
      );


      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchLdflda<Player>("DashDir"),
        instr => instr.MatchLdfld<Vector2>("Y"),
        instr => instr.MatchLdcR4(0f),
        // instr => instr.MatchBgtUnS(out _),
        instr => instr.OpCode == OpCodes.Bgt_Un || instr.OpCode == OpCodes.Bgt_Un_S
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find DashDir.Y <= 0f in Player.DashCoroutine.");
        return;
      }

      if (!cursor.TryGotoNext(MoveType.Before, instr => instr.MatchStfld<Player>("Speed"))) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find stfld this.Speed in Player.DashCoroutine after DashDir.Y <= 0f.");
        return;
      }

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.DashCoroutine to mod this.Speed decay after DashDir.Y <= 0f!");

      cursor.Emit(OpCodes.Ldarg_0);
      cursor.Emit(
        OpCodes.Ldfld,
        typeof(Player).GetMethod("DashCoroutine", BindingFlags.NonPublic | BindingFlags.Instance)
          .GetStateMachineTarget()
          .DeclaringType.GetField("<>4__this")
      );
      cursor.EmitDelegate<Func<Vector2, Player, Vector2>>(
        (newSpeed, player) => {
          if (player.Get<UnboundedDashing>() == null) { return newSpeed; }

          // var dd = DynamicData.For(player);
          // var beforeDashSpeed = dd.Get<Vector2>("beforeDashSpeed");
          // if (newSpeed.X == beforeDashSpeed.X) {
          //   var speed = this.CorrectDashPrecision(vector);
          //   return new Vector2(newSpeed.X)
          // }
          return player.Speed * (160f / 240f);
          // return UnboundedDashing.CombineDash(player.DashDir * 160f, player.Speed, 240f);
        }
      );


      ILLabel label = null;
      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchLdflda<Player>("Speed"),
        instr => instr.MatchLdfld<Vector2>("Y"),
        instr => instr.MatchLdcR4(0f),
        instr => instr.MatchBgeUn(out label)
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find Speed.Y < 0f in Player.DashCoroutine.");
        return;
      }

      // Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Modding Speed.Y < 0f to always fail in Player.DashCoroutine as Speed.Y < 0f && false");
      // cursor.Emit(OpCodes.Ldarg_0);
      // cursor.EmitDelegate<Func<Player, bool>>(
      //   (player) => {
      //     return player.Get<UnboundedDashing>() != null;
      //   }
      // );
      // cursor.Emit(OpCodes.Brtrue, label);
      // fixme: infinite loop wtf?
    }


    private static Vector2 SuperJumpOldSpeed;
    private static void ILPlayerSuperJump(ILContext il) {
      var cursor = new ILCursor(il);

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.SuperJump at start to cache this.Speed");
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Action<Player>>((player) => {
        SuperJumpOldSpeed = player.Speed;
      });

      if (!cursor.TryGotoNext(
        MoveType.Before, instr => instr.MatchStfld<Player>("varJumpSpeed")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find stfld this.varJumpSpeed in Player.SuperJump");
        return;
      }

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.SuperJump to mod this.Speed");
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Action<Player>>(
        (player) => {
          if (player.Get<UnboundedDashing>() == null) { return; }

          var staticSpeed = player.Speed;
          var newSpeed = SuperJumpOldSpeed;

          if (newSpeed.X * staticSpeed.X <= 0f) {
            newSpeed.X *= -1f;
          }

          player.Speed = UnboundedDashing.CombineDash(staticSpeed, newSpeed, staticSpeed.Length());
        }
      );
    }


    private static Vector2 SuperWallJumpOldSpeed;
    private static void ILPlayerSuperWallJump(ILContext il) {
      var cursor = new ILCursor(il);

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.SuperWallJump at start to cache this.Speed");
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Action<Player>>((player) => {
        SuperWallJumpOldSpeed = player.Speed;
      });


      if (!cursor.TryGotoNext(
        MoveType.Before, instr => instr.MatchStfld<Player>("varJumpSpeed")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find stfld this.varJumpSpeed in Player.SuperWallJump");
        return;
      }

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.SuperWallJump to mod this.Speed");
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Action<Player>>(
        (player) => {
          if (player.Get<UnboundedDashing>() == null) { return; }

          var staticSpeed = player.Speed;
          var newSpeed = SuperWallJumpOldSpeed;

          if (newSpeed.X * staticSpeed.X <= 0f) {
            newSpeed.X *= -1f;
          }

          player.Speed = UnboundedDashing.CombineDash(staticSpeed, newSpeed, staticSpeed.Length());
        }
      );
    }


    private static void ILPlayerDreamDashBegin(ILContext il) {
      var cursor = new ILCursor(il);

      if (!cursor.TryGotoNext(
        MoveType.Before, instr => instr.MatchStfld<Player>("Speed")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Hook failed to find stfld this.Speed in Player.DreamDashBegin");
        return;
      }

      Logger.Log("bitsbolts.UnboundedDashing", $"IL@{cursor.Index}: Adding delegate in Player.DreamDashBegin to mod this.Speed");
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Func<Vector2, Player, Vector2>>(
        (dreamSpeed, player) => {
          if (player.Get<UnboundedDashing>() == null) { return dreamSpeed; }

          return UnboundedDashing.CombineDash(dreamSpeed, player.Speed, dreamSpeed.Length());
        }
      );
    }


    private static void ILPlayerSuperBounce(ILContext il) {
      var cursor = new ILCursor(il);
    }


    private static void ILPlayerSideBounce(ILContext il) {
      var cursor = new ILCursor(il);
    }
  }
}
