using System;
using System.Linq;
using System.Reflection;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;

using Bitsbolts.Components;


namespace Bitsbolts {
  public static class ModCompat {
    [OnLoad]
    public static void Load() {
      On.Monocle.Entity.Added += OnEntityAdded;
    }

    [OnUnload]
    public static void Unload() {
      On.Monocle.Entity.Added -= OnEntityAdded;
    }

    private static void OnEntityAdded(On.Monocle.Entity.orig_Added orig, Entity entity, Scene scene) {
      orig(entity, scene);

      // TriggerTrigger stores it's Nodes as offsets from it's base Position,
      // which, when resized by ResizeTriggers, will neglect changing Nodes.
      // Thankfully, TriggerTrigger also locates and caches the trigger(s) that
      // it targets in Awake, so if we don't Resize it, it can correctly target
      // triggers, and we can resize it afterwards.
      if (entity.GetType().FullName == "vitmod.TriggerTrigger") {
        entity.Add(new AwakePriority(-15));
      }
    }
  }
}
