using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;


namespace Bitsbolts {
  internal static class EnumeratorExt {
    internal struct EnumerableEnumerator<E, I> : IEnumerable<I>, IEnumerable
    where E : IEnumerator<I> {
      E inner;
      public EnumerableEnumerator(E inner) { this.inner = inner; }
      IEnumerator IEnumerable.GetEnumerator() {
        return this.GetEnumerator();
      }
      public IEnumerator<I> GetEnumerator() { return inner; }
    }

    internal static IEnumerable<T> Enumerable<T>(this IEnumerator<T> enumerator) {
      return new EnumerableEnumerator<IEnumerator<T>, T>(enumerator);
    }
    // internal static IEnumerator<T> GetEnumerator<T>(this IEnumerator<T> enumerator) => enumerator;

    internal static int IndexOf<T>(this IEnumerable<T> enumerator, Func<T, bool> func) {
      var i = 0;
      foreach (var item in enumerator) {
        if (func.Invoke(item)) { return i; }
        i++;
      }

      return -1;
    }

    internal static IEnumerable<(int, T)> Enumerate<T>(this IEnumerable<T> enumerator) {
      return enumerator.Select((T item, int index) => (index, item));
    }
  }
}
