using System;
using System.Collections.Generic;
using System.Reflection;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.RuntimeDetour;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts.Components {
  public static class FancyTextExt {
    private static ILHook ILFancyTextParseHook;
    private static Hook OnFancyTextParseHook;

    [OnLoad]
    public static void OnLoad() {
      FancyTextExt.ILFancyTextParseHook = new ILHook(
        typeof(FancyText).GetMethod("Parse", BindingFlags.NonPublic | BindingFlags.Instance),
        FancyTextExt.ILFancyTextParse
      );
      FancyTextExt.OnFancyTextParseHook = new Hook(
        typeof(FancyText).GetMethod("Parse", BindingFlags.NonPublic | BindingFlags.Instance),
        typeof(FancyTextExt).GetMethod("OnFancyTextParse", BindingFlags.NonPublic | BindingFlags.Static)
      );
    }

    [OnUnload]
    public static void OnUnload() {
      FancyTextExt.ILFancyTextParseHook.Dispose();
      FancyTextExt.OnFancyTextParseHook.Dispose();
    }


    public static float CalcLinesHeight(this FancyText.Text text) {
      var fontSize = text.Font.Get(text.BaseSize);
      float height = 0f;
      float maxLineScale = 0f;

      foreach (var node in text.Nodes) {
        // uhh...? I guess only check the first page...? using this function
        // with paginated text doesn't really make any sense lol
        if (node is FancyText.NewPage) {
          break;
        } else if (node is FancyText.NewLine) {
          if (maxLineScale == 0f) { maxLineScale = 1f; }
          height += fontSize.LineHeight * maxLineScale;
          maxLineScale = 0f;
        } else if (node is FancyText.Char chr) {
          maxLineScale = Math.Max(chr.Scale, maxLineScale);
        }
      }
      height += fontSize.LineHeight * maxLineScale;
      return height;
    }

    public static float CalcLinesWidth(this FancyText.Text text) {
      return DynamicData.For(text).Get<float>("bitsbolts_maxLineWidth");
    }


    private static void ILFancyTextParse(ILContext il) {
      var cursor = new ILCursor(il);

      int localList = -69;
      if (!cursor.TryGotoNext(
        instr => instr.MatchNewobj<List<string>>(),
        instr => instr.MatchStloc(out localList)
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.FancyTextExt", $"IL@{cursor.Index}: Hook failed to find list = new List() in FancyText.Parse");
        return;
      }
      Logger.Log("bitsbolts.FancyTextExt", $"IL@{cursor.Index}: Grabbing local variable list in FancyText.Parse after list = new List()");


      if (!cursor.TryGotoNext(
        MoveType.Before,
        instr => instr.MatchLdstr("/>>"),
        instr => instr.MatchCallvirt<string>("Equals")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.FancyTextExt", $"IL@{cursor.Index}: Hook failed to find text.Equals(\"/>>\") in FancyText.Parse");
        return;
      }
      Logger.Log("bitsbolts.FancyTextExt", $"IL@{cursor.Index}: Adding delegate branch in FancyText.Parse before text.Equals(\"/>>\")");

      // OnFancyTextParseParam(text, this, localList);
      cursor.Emit(OpCodes.Dup);  // the top of the stack should be `text`
      cursor.Emit(OpCodes.Ldarg_0);
      cursor.Emit(OpCodes.Ldloc_S, (byte) localList);
      cursor.EmitDelegate<Action<string, FancyText, List<string>>>(FancyTextExt.OnFancyTextParseParam);
      // branch is not needed? ig it falls through anyways so idc
      // cursor.Emit(OpCodes.Brtrue, end);
    }

    private static void OnFancyTextParseParam(string cmd, FancyText text, List<string> args) {
      var dd = DynamicData.For(text);
      if (cmd.Equals("^")) {
        float size = 1f;
        if (args.Count > 0) {
          float.TryParse(
            args[0],
            System.Globalization.NumberStyles.Float,
            System.Globalization.CultureInfo.InvariantCulture,
            out size
          );
        }
        dd.Set("currentScale", size);
      } else if (cmd.Equals("/^")) {
        dd.Set("currentScale", 1f);
      }
    }

    delegate FancyText.Text OnFancyTextParse_orig(FancyText self);
    private static FancyText.Text OnFancyTextParse(OnFancyTextParse_orig orig, FancyText self) {
      var text = orig(self);
      DynamicData.For(text).Set(
        "bitsbolts_maxLineWidth",
        (float) DynamicData.For(self).Get<int>("maxLineWidth")
      );
      return text;
    }
  }
}
