using System;
using System.Reflection;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts {
  internal class MonocleExt {
    internal static void TrackEntity<T>(bool inherited = false) {
      TrackEntity(typeof(T), inherited);
    }

    internal static void TrackEntity(Type type, bool inherited = false) {
      // wtf it doesn't inherit from entity? why?
      if (!typeof(Entity).IsAssignableFrom(type)) { return; }
      // already tracked; don't track it again
      if (Tracker.StoredEntityTypes.Contains(type)) { return; }

      Logger.Log("bitsbolts.MonocleExt", $"Adding entity type {type.FullTypeName()} to tracker.");

      if (!type.IsAbstract) {
        if (!Tracker.TrackedEntityTypes.ContainsKey(type)) {
          Tracker.TrackedEntityTypes.Add(type, new List<Type>());
        }
        Tracker.TrackedEntityTypes[type].Add(type);
      }
      Tracker.StoredEntityTypes.Add(type);

      if (inherited) {
        foreach (var subtype in EnumerateSubclasses(type).Enumerable()) {
          if (!subtype.IsAbstract) {
            if (!Tracker.TrackedEntityTypes.ContainsKey(subtype)) {
              Tracker.TrackedEntityTypes.Add(subtype, new List<Type>());
            }
            Tracker.TrackedEntityTypes[subtype].Add(type);
          }
        }
      }
    }

    // todo: TrackComponent...

    private static IEnumerator<Type> EnumerateSubclasses(Type type) {
      foreach (Type subtype in Assembly.GetEntryAssembly().GetTypes()) {
        if (type != subtype && type.IsAssignableFrom(subtype)) {
          yield return subtype;
        }
      }
      yield break;
    }
  }
}
