using System;
using System.Collections.Generic;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod.Entities;

using Flagstr = System.Tuple<string, bool>;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts {
  internal static class Parsing {
    internal class ParseException : Exception {
      internal ParseException() {}
      internal ParseException(string message) : base(message) {}
      internal ParseException(string message, Exception inner) : base(message, inner) {}
    }


    internal static EntityID ParseEID(this string input) {
      try {
        var eid = new EntityID("", 0);
        eid.Key = input;
        return eid;
      } catch (IndexOutOfRangeException ex) {
        throw new ParseException($"Parsing EntityID encountered invalid input: {input}", ex).Carded();
      }
    }

    internal static List<EntityID> ParseEIDs(this string input) {
      return input
        .Split(new char[] { ',' }, System.StringSplitOptions.RemoveEmptyEntries)
        .Select(Parsing.ParseEID)
        .ToList();
    }


    internal static (string Flag, bool Enable) ParseFlagstr(
      this string input, bool allowInversions = true
    ) {
      if (input.Length >= 1 && input[0] == '!') {
        if (!allowInversions) {
          throw new ParseException($"Flag {input} cannot be inverted in this context!").Carded();
        }
        return (input.Substring(1), false);
      }
      return (input, true);
    }

    internal static Dictionary<string, bool> ParseFlagstrs(
      this string input, bool allowInversions = true
    ) {
      return input
        .Split(new char[] { ',' }, System.StringSplitOptions.RemoveEmptyEntries)
        .Select(str => Parsing.ParseFlagstr(str, allowInversions))
        .ToDictionary(flag => flag.Flag, flag => flag.Enable);
    }

    internal static Flagstrs InvertFlagstrs(this Flagstrs flags) {
      var output = new Flagstrs(flags);
      foreach (var flag in flags) {
        output[flag.Key] = !flag.Value;
      }
      return output;
    }

    internal static void SetFlagstrs(this Session session, Dictionary<string, bool> flags) {
      foreach (var flag in flags) {
        session.SetFlag(flag.Key, flag.Value);
      }
    }

    internal static Dictionary<string, bool> GetFlagstrs(this Session session, Dictionary<string, bool> flags) {
      var output = new Flagstrs(flags);
      foreach (var flag in flags) {
        output[flag.Key] = session.GetFlag(flag.Key) == flag.Value;
      }
      return output;
    }


    internal static HashSet<string> ParseLevelsList(this string input, MapData map) {
      // todo: completely reimplement this ourselves to add inversion support
      return (HashSet<string>) DynamicData.For(map).Invoke("ParseLevelsList", input);
    }
  }
}
