using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;


namespace Bitsbolts {
  public struct SerializedEntity<T> : IYamlConvertible {
    public string Name;
    public T Data;

    // get out of our way type system! we're going to reinterpret these bits or die trying
    public static explicit operator SerializedEntity<object>(SerializedEntity<T> self) {
      return new SerializedEntity<object> { Name = self.Name, Data = (object) self.Data };
    }

    public static explicit operator SerializedEntity<T>(SerializedEntity<object> self) {
      return new SerializedEntity<T> { Name = self.Name, Data = (T) self.Data };
    }

    void IYamlConvertible.Read(IParser parser, Type expected, ObjectDeserializer nested) {
      parser.Consume<MappingStart>();

      // fuck you if you reorder these fields
      parser.Consume<Scalar>();
      this.Name = (string) nested(typeof(string));

      if (SessionEntity.Restorers.ContainsKey(this.Name)) {
        var restorer = SessionEntity.Restorers[this.Name];

        parser.Consume<Scalar>();
        this.Data = (T) nested(restorer.DataType);
      } else {
        throw new KeyNotFoundException($"Entity restorer for data {this.Name} is not registered!").Carded();
      }

      parser.Consume<MappingEnd>();
    }

    void IYamlConvertible.Write(IEmitter emitter, ObjectSerializer nested) {
      emitter.Emit(new MappingStart());

      emitter.Emit(new Scalar("name"));
      nested(this.Name, typeof(string));

      emitter.Emit(new Scalar("data"));
      nested(this.Data, this.Data.GetType());

      emitter.Emit(new MappingEnd());
    }
  }

  public class SerializedEntityLookup : Dictionary<EntityID, SerializedEntity<object>> {
    public SerializedEntityLookup() : base() {}
    public SerializedEntityLookup(int capacity) : base(capacity) {}

    public SerializedEntity<T> Get<T>(EntityID key) { return (SerializedEntity<T>) base[key]; }
    public void Set<T>(EntityID key, SerializedEntity<T> value) { base[key] = (SerializedEntity<object>) value; }
  }
}
