using System;
using System.Collections;
using System.Reflection;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using Celeste.Mod;


namespace Bitsbolts.Scenes {
  public class CrashCard : Scene {
    internal static void Show(CardedException ex) {
      Logger.LogDetailed(ex);
      Engine.Scene = new CrashCard(ex);
    }
    private static bool JustShowed = false;

    private Postcard postcard;
    private CrashCard(CardedException ex) {
      if (CrashCard.JustShowed) {
        throw new Exception("Crash while showing crash postcard! Wow isn't that ironic huh", ex);
      }
      CrashCard.JustShowed = true;

      Audio.SetMusic(null, true, true);
      Audio.SetAmbience(null, true);


      Exception realex = ex.ToException();
      var realexName = realex.GetType().Name;

      var trace = new System.Diagnostics.StackTrace(ex);
      var stackTrace = "";
      for (int i = 0; i < Math.Min(trace.FrameCount, 5); i++) {
        var frame = trace.GetFrame(i);
        var method = frame.GetMethod();
        if (method == null) {
          stackTrace += $"at <unknown>";
        } else {
          stackTrace += $"at {method.FullTypeName()}";
        }
        stackTrace += $":{frame.GetFileLineNumber().ToString()}";
        stackTrace += "{n}";
      }

      if (stackTrace.Length > 0) {
        stackTrace = stackTrace.Substring(0, stackTrace.Length - 3);
      }

      this.postcard = new Postcard(
        "bitsbolts_oopsie".DialogNlFormat(
          realex.Message, realexName, stackTrace
        ),
        "event:/ui/main/postcard_ch5_in",
        "event:/ui/main/postcard_ch5_out"
      );

      base.Add(new Entity { new Coroutine(this.ShowRoutine()) });
      base.Add(new HudRenderer());
    }

    public IEnumerator ShowRoutine() {
      yield return 0.25f;
      base.Add(postcard);
      CrashCard.JustShowed = false;

      yield return postcard.DisplayRoutine();

      Engine.Scene = new OverworldLoader(Overworld.StartMode.AreaQuit);
      yield break;
    }

    public override void BeforeRender() {
      base.BeforeRender();
      if (this.postcard != null) { this.postcard.BeforeRender(); }
    }


    [OnLoad]
    public static void Load() {
      if (!Util.IsCore()) {
        On.Monocle.Engine.Update += TryCatchUpdate;
        On.Monocle.Engine.Draw += TryCatchDraw;
      }
    }

    [OnUnload]
    public static void Unload() {
      if (!Util.IsCore()) {
        On.Monocle.Engine.Update -= TryCatchUpdate;
        On.Monocle.Engine.Draw -= TryCatchDraw;
      }
    }

    private static void TryCatchUpdate(
      On.Monocle.Engine.orig_Update orig, Engine self, GameTime gt
    ) {
      try {
        orig(self, gt);
      } catch (CardedException ex) {
        CrashCard.Show(ex);
      }
    }

    private static void TryCatchDraw(
      On.Monocle.Engine.orig_Draw orig, Engine self, GameTime gt
    ) {
      try {
        orig(self, gt);
      } catch (CardedException ex) {
        CrashCard.Show(ex);
      }
    }
  }
}
