using System;
using System.Linq;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using Mono.Cecil.Cil;
using MonoMod.Cil;
using MonoMod.RuntimeDetour;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using Bitsbolts.Components;


namespace Bitsbolts.Triggers {
  [Tracked]
  [CustomEntity("Bitsbolts/BlockTransition")]
  public class BlockTransition : Trigger {
    [OnLoad]
    public static void Load() {
      IL.Celeste.Level.EnforceBounds += ILLevelEnforceBounds;
    }

    [OnUnload]
    public static void Unload() {
      IL.Celeste.Level.EnforceBounds -= ILLevelEnforceBounds;
    }


    public bool BlockUp;
    public bool BlockDown;
    public bool BlockLeft;
    public bool BlockRight;

    public BlockTransition(EntityData data, Vector2 offset) : base(data, offset) {
      this.BlockUp = data.Bool("blockUp", false);
      this.BlockDown = data.Bool("blockDown", false);
      this.BlockLeft = data.Bool("blockLeft", false);
      this.BlockRight = data.Bool("blockRight", false);

      var flags = data.Attr("flags").ParseFlagstrs();
      if (flags.Count > 0) { base.Add(new FlagstrTrigger(flags)); }
      base.Add(new TransitionTrigger(before: false));
    }

    private static void ILLevelEnforceBounds(ILContext il) {
      var cursor = new ILCursor(il);

      // left
      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchCallvirt<MapData>("CanTransitionTo")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.Triggers.BlockTransition", $"IL@{cursor.Index}: Hook failed to find first CanTransitionTo in Level.EnforceBounds");
        return;
      }

      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Func<bool, Level, bool>>(
        (bool doTransition, Level level) => {
          return !level.Tracker.GetEntities<BlockTransition>().Any(
            entity => entity is BlockTransition tr && tr.PlayerIsInside && tr.BlockLeft
          ) && doTransition;
        }
      );


      // right
      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchCallvirt<MapData>("CanTransitionTo")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.Triggers.BlockTransition", $"IL@{cursor.Index}: Hook failed to find second CanTransitionTo in Level.EnforceBounds");
        return;
      }

      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Func<bool, Level, bool>>(
        (bool doTransition, Level level) => {
          return !level.Tracker.GetEntities<BlockTransition>().Any(
            entity => entity is BlockTransition tr && tr.PlayerIsInside && tr.BlockRight
          ) && doTransition;
        }
      );


      // up
      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchCallvirt<MapData>("CanTransitionTo")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.Triggers.BlockTransition", $"IL@{cursor.Index}: Hook failed to find third CanTransitionTo in Level.EnforceBounds");
        return;
      }

      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Func<bool, Level, bool>>(
        (bool doTransition, Level level) => {
          return !level.Tracker.GetEntities<BlockTransition>().Any(
            entity => entity is BlockTransition tr && tr.PlayerIsInside && tr.BlockUp
          ) && doTransition;
        }
      );


      // down
      if (!cursor.TryGotoNext(
        MoveType.After,
        instr => instr.MatchCallvirt<MapData>("CanTransitionTo")
      )) {
        Logger.Log(LogLevel.Error, "bitsbolts.Triggers.BlockTransition", $"IL@{cursor.Index}: Hook failed to find fourth CanTransitionTo in Level.EnforceBounds");
        return;
      }

      cursor.Emit(OpCodes.Ldarg_0);
      cursor.EmitDelegate<Func<bool, Level, bool>>(
        (bool doTransition, Level level) => {
          return !level.Tracker.GetEntities<BlockTransition>().Any(
            entity => entity is BlockTransition tr && tr.PlayerIsInside && tr.BlockDown
          ) && doTransition;
        }
      );
    }
  }
}
