using System;
using System.Linq;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;
using ControllingPlaybackComponent = Bitsbolts.Components.ControllingPlayback;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/ControllingPlayback")]
  [Tracked]
  public class ControllingPlayback : Trigger {
    public List<Player.ChaserState> Timeline;

    public ControllingPlayback(EntityData data, Vector2 offset, EntityID eid) : base(data, offset) {
      var tutorial = data.Attr("tutorial", "");
      if (!PlaybackData.Tutorials.ContainsKey(tutorial)) {
        throw new KeyNotFoundException($"Tutorial {tutorial} was not found for controlling playback eid {eid}!").Carded();
      }
      this.Timeline = PlaybackData.Tutorials[tutorial];

      var flags = data.Attr("flags", "").ParseFlagstrs();
      if (flags.Count > 0) { base.Add(new FlagstrTrigger(flags)); }
      // the playback should start immediately on first update
      base.Add(new TransitionTrigger(before: false));
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);

      if (player.Get<ControllingPlaybackComponent>() == null) {
        player.Add(new ControllingPlaybackComponent(this.Timeline));
      }
    }
  }
}
