using System;

using Microsoft.Xna.Framework;
using Monocle;
using Celeste;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/FlagLookoutBlocker")]
  [TrackedAs(typeof(LookoutBlocker))]
  public class FlagLookoutBlocker : LookoutBlocker {
    private Collider origCollider;

    public readonly Flagstrs Flags;
    public FlagLookoutBlocker(EntityData data, Vector2 offset) : base(data, offset) {
      this.Flags = data.Attr("flags", "").ParseFlagstrs();
    }

    public override void Awake(Scene scene) {
      // backup the collider, but this needs to be done in awake just in case
      // the collider was resized or modified by other triggers before awake
      origCollider = base.Collider;

      // FlagstrTrigger like only works on Triggers man... use FlagstrCollidable
      // The component typically activates in Added, however since we need to wait
      // until awake anyways, delay it til now
      if (this.Flags.Count > 0) {
        base.Add(new FlagstrCollidable(this.Flags, this.OnChange));
      }

      // the base calls EntityAwake on components, and since we're adding this
      // component so late, we need to call base after adding everything
      base.Awake(scene);
    }

    public void OnChange(bool collidable) {
      if (collidable) {
        base.Collider = origCollider;
      } else {
        // so like Lookout only sorta respects Entity.Collideable: it still
        // grabs all tracked LookoutBlockers and uses them for checking bounds
        // manually? fuck it make sure any check will always fail

        // I swear if someone puts a room over here...
        base.Collider = new Hitbox(0f, 0f, -1e38f, -1e38f);
      }
    }
  }
}
