using System;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/FlagZone")]
  public class FlagZone : Trigger {
    public readonly Flagstrs Flags;
    public readonly bool UnsetLevelStart = false;

    public FlagZone(EntityData data, Vector2 offset) : base(data, offset) {
      this.Flags = data.Attr("flags").ParseFlagstrs();
      this.UnsetLevelStart = data.Bool("unsetLevelStart", false);

      // the transition OnEnter may occur after Awake, overwriting flags
      base.Add(new TransitionTrigger(before: true));
    }

    public override void Awake(Scene scene) {
      base.Awake(scene);
      if (this.UnsetLevelStart) {
        (scene as Level).Session.SetFlagstrs(this.Flags.InvertFlagstrs());
      }
    }

    public override void Removed(Scene scene) {
      // In case the player dies inside the trigger it's probably best we still unset it...
      (scene as Level).Session.SetFlagstrs(this.Flags.InvertFlagstrs());
      base.Removed(scene);
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);
      (this.Scene as Level).Session.SetFlagstrs(this.Flags);
    }

    public override void OnLeave(Player player) {
      base.OnLeave(player);
      (this.Scene as Level).Session.SetFlagstrs(this.Flags.InvertFlagstrs());
    }
  }
}
