using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/Flags")]
  [Tracked]
  public class Flags : Trigger {
    public Flagstrs SetFlags;
    public Flags.Events Event;

    public Flags(EntityData data, Vector2 offset) : base(data, offset) {
      this.Event = (Events) data.Int("event");
      this.SetFlags = data.Attr("flags", "").ParseFlagstrs();
      base.Add(new TransitionTrigger(before: false));
    }

    public override void Added(Scene scene) {
      base.Added(scene);
      if (this.Event == Events.OnAdded) {
        (this.Scene as Level).Session.SetFlagstrs(this.SetFlags);
      }
    }

    public override void Removed(Scene scene) {
      if (this.Event == Events.OnRemove) {
        (this.Scene as Level).Session.SetFlagstrs(this.SetFlags);
      }
      base.Removed(scene);
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);
      if (this.Event == Events.OnEnter) {
        (this.Scene as Level).Session.SetFlagstrs(this.SetFlags);
      }
    }

    public override void OnLeave(Player player) {
      base.OnLeave(player);
      if (this.Event == Events.OnLeave) {
        (this.Scene as Level).Session.SetFlagstrs(this.SetFlags);
      }
    }

    public enum Events {
      OnEnter,
      OnLeave,
      OnAdded,
      OnRemove,
    }
  }
}
