using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/NoRetry")]
  [Tracked]
  public class NoRetry : Trigger {
    public NoRetry(EntityData data, Vector2 offset) : base(data, offset) {
      var flags = data.Attr("flags", "").ParseFlagstrs();
      if (flags.Count > 0) { base.Add(new FlagstrTrigger(flags)); }
      base.Add(new TransitionTrigger(before: false));
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);
      (base.Scene as Level).CanRetry = false;
    }

    public override void OnStay(Player player) {
      base.OnStay(player);
      (base.Scene as Level).CanRetry = false;
    }

    public override void OnLeave(Player player) {
      base.OnLeave(player);
      (base.Scene as Level).CanRetry = true;
    }


    public override void Removed(Scene scene) {
      // Only reset it if there's no other copy of ourselves controlling it.
      // This also counts entities loaded during a transition, because the
      // entity in the next room will take it over for us.
      if (
        scene.Tracker.GetEntities<NoRetry>().All(trig => !(trig as NoRetry).Triggered)
      ) {
        (scene as Level).CanRetry = true;
      }
      base.Removed(scene);
    }
  }
}
