using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/RedirectMotion")]
  public class RedirectMotion : Trigger {
    public readonly Flagstrs Flags;
    public readonly Modes Mode;

    public RedirectMotion(EntityData data, Vector2 offset) : base(data, offset) {
      this.Flags = data.Attr("flags", "").ParseFlagstrs();
      this.Mode = (Modes) data.Int("mode", 0);

      if (this.Flags.Count > 0) { base.Add(new FlagstrTrigger(this.Flags)); }
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);

      // Vector2 old_dir = Vector2.Normalize(player.Speed);
      float old_mag = player.Speed.Length();

      Vector2 new_dir = Vector2.Zero;
      switch (this.Mode) {
        case Modes.Up: new_dir = new Vector2(0, -1); break;
        case Modes.UpRight: new_dir = new Vector2(1, -1); break;
        case Modes.Right: new_dir = new Vector2(1, 0); break;
        case Modes.DownRight: new_dir = new Vector2(1, 1); break;
        case Modes.Down: new_dir = new Vector2(0, 1); break;
        case Modes.DownLeft: new_dir = new Vector2(-1, 1); break;
        case Modes.Left: new_dir = new Vector2(-1, 0); break;
        case Modes.UpLeft: new_dir = new Vector2(-1, -1); break;
      }
      new_dir.Normalize();

      player.Speed = new_dir * old_mag;

      if (new_dir.X > 0f) {
        player.Facing = Facings.Right;
      } else if (new_dir.X < 0f) {
        player.Facing = Facings.Left;
      }
      // don't change Facing if it's just vertical motion; looks off
    }

    public enum Modes {
      Up, UpRight, Right, DownRight, Down, DownLeft, Left, UpLeft
    }
  }

}
