using System;
using System.Linq;

using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using Celeste;
using Celeste.Mod;
using Celeste.Mod.Entities;

using Bitsbolts.Components;
using Flagstrs = System.Collections.Generic.Dictionary<string, bool>;


namespace Bitsbolts.Triggers {
  [CustomEntity("Bitsbolts/SkipBadelineBoost")]
  public class SkipBadelineBoost : Trigger {
    [OnLoad]
    public static void Load() {
      On.Monocle.Tracker.Initialize += TrackerInitialize;
    }

    [OnUnload]
    public static void Unload() {
      On.Monocle.Tracker.Initialize -= TrackerInitialize;
    }

    private static void TrackerInitialize(On.Monocle.Tracker.orig_Initialize orig) {
      orig();
      MonocleExt.TrackEntity<BadelineBoost>(inherited: true);
    }


    public readonly int Index;
    public readonly bool Instant;
    public readonly bool Backward;
    public readonly Vector2 FindPos;
    private BadelineBoost target = null;

    public SkipBadelineBoost(EntityData data, Vector2 offset) : base(data, offset) {
      this.Index = data.Int("index");
      this.Instant = data.Bool("instant", false);
      this.Backward = data.Bool("backward", false);

      this.FindPos = data.Nodes.Length > 0 ? data.NodesOffset(offset)[0] : this.Position;

      var flags = data.Attr("flags", "").ParseFlagstrs();
      if (flags.Count > 0) { base.Add(new FlagstrTrigger(flags)); }
    }

    public override void Awake(Scene scene) {
      base.Awake(scene);
      target = scene.Tracker.GetNearestEntity<BadelineBoost>(this.FindPos);
      if (target == null) {
        Logger.Log(LogLevel.Warn, "bitsbolts.SkipBadelineBoost", $"Badeline boost entity not found in level! Removing self.");
        base.RemoveSelf();
      }
    }

    public override void OnEnter(Player player) {
      base.OnEnter(player);

      if (target != null && target.Scene == base.Scene) {
        var dd = DynamicData.For(target);
        var travelling = dd.Get<bool>("travelling");
        var curIndex = dd.Get<int>("nodeIndex");
        var nodes = dd.Get<Vector2[]>("nodes");

        if (this.Index < 0 || this.Index >= nodes.Length) {
          Logger.Log(LogLevel.Error, "bitsbolts.SkipBadelineBoost", $"Provided index {this.Index} is out of bounds! Entity max index is {nodes.Length}!");
          return;
        }

        if (
          travelling
          || this.Index == curIndex  // don't bother; it won't do shit anyways
          || (this.Index < curIndex && !this.Backward)  // don't go backwards unless explicitly set
        ) { return; }

        if (this.Instant) {
          dd.Set("nodeIndex", this.Index);
          target.Position = nodes[this.Index];
        } else {
          // make Skip think we are at the node just before the desired node
          // it uses entity position in the tween instead of the last node
          // Skip never actually accesses nodeIndex[-1] so it's fine
          dd.Set("nodeIndex", this.Index - 1);
          dd.Invoke("Skip");
        }
      }
    }
  }
}
